<?php

// Making sure the access to all server files will be done from this file only
define('AUTH_ACCESS', TRUE);

require_once 'OngageConnect.php';

try
{
	// Initialize Ongage client
	$Ongage_Client  = new OngageConnect();

	// This should not happen but just in case
	if ( ! OngageConnect::array_get($_SERVER, 'REQUEST_METHOD'))
		throw new Exception('Unidentified request method');

	// Extract request payload
	$request_payload = $_SERVER['REQUEST_METHOD'] === 'POST'
		? json_decode(OngageConnect::array_get($_POST,'request_payload', []), TRUE)
		: $_GET;

	// Get mandatory variables
	$contact_id = OngageConnect::array_get($request_payload, 'contact_id');
	$email      = OngageConnect::array_get($request_payload, 'email');
	$list_id    = OngageConnect::array_get($request_payload, 'list_id');


	//--- Initial validations -----------------------------------------/

	// This SDK is depended on CURL library - if you don't have it installed in your local server - please install it
	// See http://php.net/manual/en/curl.installation.php
	//
	// How to install on Linux (with Apache server) [Taken from https://www.digitalocean.com/community/questions/curl-is-not-installed-in-your-php-installation]:
	//
	// 1. First Install CURL by typing "sudo apt-get install curl"
	// 2. Then Restart Apache by typing "sudo service apache2 restart"
	// 3. Then Install PHP5 CURL by typing "sudo apt-get install php5-curl"
	// 4. will prompt to install... type "y" or yes!
	// 5. Then Restart Apache by typing "sudo service apache2 restart"
	// 6. Done!
	if ( ! extension_loaded('curl'))
		throw new Exception('Pleas install CURL in your local server');

	if ( ! in_array($_SERVER['REQUEST_METHOD'], [ 'POST', 'GET' ]))
		throw new Exception('Invalid HTTP request');

	if ( ! $email)
		throw new Exception('Contact email is missing');

	if ( ! $contact_id)
		throw new Exception('Contact id is missing');

	if ( ! $list_id)
		throw new Exception('Contact list id is missing');

	// Always load the client to validate that he exists and that the right credentials were given
	$contact = $Ongage_Client->get_contact($email, $list_id, $contact_id);

	// Deny access if contact is banned or deleted
	if (in_array(OngageConnect::array_get($contact, 'ocx_status'), [ 'Banned', 'Deleted' ]))
		throw new Exception('Contact was not found');


	//--- Handel request according to the HTTP type ------------------------------------/

	$response = [
		'error'   => FALSE,
		'message' => NULL,
		'payload' => []
	];

	switch ($_SERVER['REQUEST_METHOD'])
	{
		case 'GET':

			// Get list fields of contact list
			$list_fields = $Ongage_Client->get_list_fields($list_id);

			// Create form inputs dynamically
			$response['payload'] = [
				'form_inputs' => $Ongage_Client->prepare_preference_form_inputs($contact, $list_fields),
				'contact'     => $contact
			];
			$response['message'] = 'Contact was fetched successfully';

			break;

		case 'POST':

			$action = OngageConnect::array_get($request_payload, 'action');
			switch ($action)
			{
				case 'unsubscribe':
				case 'resubscribe':
					$response['payload'] = $Ongage_Client->{$action}($email, $list_id);
					$response['message'] = 'You have been successfully '.$action.'d, pleas note that it might take few minuets until our system will be will be fully updated';
					break;
				case 'delete':
					$response['payload'] = $Ongage_Client->delete_contact($contact_id, $list_id);
					$response['message'] = 'You have been deleted successfully from our system, pleas note that it might take few minuets until our system will be fully updated';
					break;
				case 'update':

					// Extract update request payload
					$update_payload = OngageConnect::array_get($request_payload, 'update_payload');

					if ( ! $update_payload)
						throw new Exception('Update payload is missing from the request');

					// Extract fields for update
					$list_fields      = $Ongage_Client->get_list_fields($list_id);
					$fields_to_update = OngageConnect::array_extract_exist($update_payload, array_keys($list_fields));
					// Update the contact and set the response
					$response['payload'] = $Ongage_Client->update_contact($email, $list_id, $fields_to_update);
					$response['message'] = 'Your details were updated successfully';
					break;
			}

			break;
	}

	// Set success headers and print the JSON response
	http_response_code(200);
	header('Content-Type: application/json');

	echo json_encode($response);
}
//--- Catch Ongage APi errors ----------------------------------------------------------/
catch (Ongage_Api_Error $e)
{
	$http_code = $e->getCode();
	http_response_code($http_code);
	header('Content-Type: application/json');
	echo json_encode(
		[
			'error'          => TRUE,
			'message'        => $e->getMessage(),
			'errors_payload' => $e->get_errors()
		]
	);
}
//--- Catch Ongage SDK errors ----------------------------------------------------------/
catch (Ongage_SDK_error $e)
{
	http_response_code(500);
	header('Content-Type: application/json');
	echo json_encode(
		[
			'error'          => TRUE,
			'message'        => $e->getMessage(),
			'errors_payload' => []
		]
	);
}
//--- Catch general errors -------------------------------------------------------------/
catch (Exception $e)
{
	// Request didn't pass the validations - send the error in a json back to the form page where it will be displayed
	// Note that you can alter this if you don't want to show validation error
	http_response_code(500);
	header('Content-Type: application/json');
	echo json_encode(
		[
			'error'          => TRUE,
			'message'        => $e->getMessage(),
			'errors_payload' => []
		]
	);
}

